package org.openxma.dsl.generator.output;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.xpand2.output.FileHandleImpl;
import org.eclipse.xpand2.output.Outlet;
import org.eclipse.xtext.util.StringInputStream;

public class EclipseResourceFileHandle extends FileHandleImpl {
    private final Log log = LogFactory.getLog(getClass());
    private IFile file;

    public EclipseResourceFileHandle(Outlet outlet, IFile file) {
        super(outlet, file.getLocation().toFile());
        this.file = file;
    }

    @Deprecated
    public File getTargetFile() {
        return super.getTargetFile();
        // the HbmFileMergePostProcessor cannot deal with the previously thrown Exception
        //  -->
        // throw new UnsupportedOperationException();
    }

    public String getFileEncoding() {
        try {
            return file.getCharset();
        } catch (CoreException e) {
            return "UTF-8";
        }
    }

    public void writeAndClose() {
        try {
            if (!isOverwrite() && file.exists()) {
                if (log.isDebugEnabled()) {
                    log.debug("Skipping file : " + file.getLocation() + " cause it exists already");
                }
                return;
            }
            if (log.isDebugEnabled()) {
                log.debug("Opening file : " + file.getLocation());
            }
            if (getOutlet().shouldWrite(this)) {
                getOutlet().beforeWriteAndClose(this);
                if (file.exists()) {
                    file.setContents(new StringInputStream(getBuffer().toString(),getFileEncoding()), true, false,
                            new NullProgressMonitor());
                } else {
                    ensureExists(file.getParent());
                    file.create(new StringInputStream(getBuffer().toString(),getFileEncoding()), true, new NullProgressMonitor());
                }

            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            getOutlet().afterClose(this);
        }

    }

    protected void ensureExists(IContainer parent) {
        if (parent instanceof IFolder && !parent.exists()) {
            ensureExists(parent.getParent());
            try {
                ((IFolder) parent).create(true, true, null);
            } catch (CoreException e) {
                throw new RuntimeException(e);
            }
        }
    }

}
